# -*- coding: utf-8 -*-

import os
import re
from collections import OrderedDict

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.template import Lexer, Parser
from django.template.defaulttags import CommentNode


STYLEGUIDE_DEBUG = getattr(settings, 'STYLEGUIDE_DEBUG', settings.DEBUG)
STYLEGUIDE_CACHE_NAME = getattr(settings, 'STYLEGUIDE_CACHE_NAME', 'styleguide_components')
STYLEGUIDE_DIR_NAME = getattr(settings, 'STYLEGUIDE_DIR_NAME', 'styleguide')
STYLEGUIDE_IGNORE_FOLDERS = getattr(settings, 'STYLEGUIDE_IGNORE_FOLDERS', ('includes', ))

FILE_NAME_RE = re.compile('^\d{2}\-')


class Styleguide(object):
    """ Main class which is delivered to template """
    
    def __init__(self):
        self._modules = None
        self._items = None
        self.current_module = None
        self._loader = StyleguideLoader()


    @property
    def modules(self):
        if self._modules is None:
            self._modules = []
            for name, components in self._loader.get_styleguide_components().items():
                module_id = name.replace(' ', '_')
                module = {
                    'id': module_id,
                    'name': name,
                    'link': reverse("styleguide.module", args=(module_id, )),
                    'components': [StyleguideComponent(c) for c in components]
                }
                self._modules.append(StyleguideModule(module))

        return self._modules


    @property
    def components(self):
        ret = []
        for module in self.modules:
            ret += module.components

        return ret

    @property
    def items(self):
        """
        :deprecated: 
        For retro compatibility only
        """
        return self._loader.get_styleguide_components().items()


    @property
    def current_components(self):
        if self.current_module is None:
            return []
        else:
            return self.current_module.components


    def set_current_module(self, module_name):
        """ Sets the given module as the current one """
        for module in self.modules:
            if module.name == module_name:
                self.current_module = module
                break


    def is_index(self):
        """ If a module is defined as current, 
        we say that we are not in an index page """
        return self.current_module is None


class StyleguideComponent(dict):

    def __init__(self, data):
        self._data = data

    def __repr__(self):
        return self._data.__str__()

    def to_dict(self):
        return self._data

    @property
    def id(self):
        return self._data['id']

    @property
    def name(self):
        return self._data['name']

    @property
    def link(self):
        return self._data['link']

    @property
    def doc(self):
        return self._data['doc']


class StyleguideModule(StyleguideComponent):
    
    @property
    def components(self):
        return self._data['components']


class StyleguideLoader(object):


    def _get_template_dirs(self):
        """
        Returns all template folder from all installed apps
        -> tuple(string, string, ..)
        """

        # Based on: https://github.com/django/django/blob/master/django/template/loaders/app_directories.py
        styleguide_template_dirs = []

        try:
            from django.template.loaders.app_directories import app_template_dirs
        except ImproperlyConfigured:
            # Running outside django's environmement
            app_template_dirs = []

        for app_template_dir in app_template_dirs:
            template_dir = os.path.join(app_template_dir, STYLEGUIDE_DIR_NAME)

            if os.path.isdir(template_dir):
                styleguide_template_dirs.append(template_dir)

        return tuple(styleguide_template_dirs)


    def get_styleguide_components(self):
        """
        Search for all templates files in `STYLEGUIDE_DIR_NAME` app_directories
        in all installed apps in the django project

        returns a dict with the folder name as the key and a list with all template
        files in that folder.

        for example, givin the following folder structure:

        /templates
          /styleguide
            /layout
              /header.html
              /footer.html
            /components
              /bar.html
              /list.html

        Would result in the following dict:

        {
            'layout' : [
                { 'id': 'header', 'name': 'header', 'file_name': 'header.html', 'template' : 'styleguide/layout/header.html', ... },
                { 'id': 'footer', 'name': 'footer', 'file_name': 'footer.html', 'template' : 'styleguide/layout/footer.html', ... },
            ],
            'components' : [
                { 'id': 'bar', 'name': 'bar', 'file_name': 'bar.html', 'template' : 'styleguide/components/bar.html', ... },
                { 'id': 'list', 'name': 'list', 'file_name': 'list.html', 'template' : 'styleguide/components/list.html', ... },
            ]
        }
        """

        ret = OrderedDict()
        styleguide_template_dirs = self._get_template_dirs()

        for styleguide_template_dir in styleguide_template_dirs:
            for root, dirs, files in os.walk(styleguide_template_dir):
                # To make it walk alphabetically
                dirs.sort()

                for dir_name in dirs:
                    if dir_name in STYLEGUIDE_IGNORE_FOLDERS:
                        continue
                    ret[dir_name] = self._get_components_from_folder(root, dir_name)

        return ret


    def _get_components_from_folder(self, root, dir_name):
        """
        :root: The whole path to the folder
        :dir_name: The folder's name

        -> dict
        """

        components = []
        for root, dirs, files in os.walk(os.path.join(root, dir_name)):
            # To make it walk alphabetically
            files.sort()

            for file_name in files:
                if not file_name.endswith('.html'):
                    # only template files
                    continue

                doc = self.get_doc_from_file(os.path.join(root, file_name))
                component_id = self._format_file_id(file_name)

                component = {
                    'id': component_id,
                    'name': self._format_file_name(doc.get('name', file_name)),
                    'file_name': file_name,
                    'template': os.path.join(STYLEGUIDE_DIR_NAME, dir_name, file_name),
                    'doc': doc,
                    'link': reverse("styleguide.component", args=(dir_name, component_id,))
                }

                components.append(component)

        return components


    def _format_file_id(self, file_name):
        """
        returns a valid string which can be used in html id attribute
        """
        return self._format_file_name(file_name).replace(' ', '-')


    def _format_file_name(self, file_name):
        """
        returns the file_name formatted to display
        """

        # if the file_name startswith two digits, remove them
        file_name = FILE_NAME_RE.split(file_name)[-1]
        return file_name.split('.html')[0].replace('_', ' ')


    def get_doc_from_file(self, file_path):
        """
        :file_path: string
        -> dict
        """

        doc_string = self.extract_doc_from_file(file_path)
        return self.parse_doc(doc_string)


    def extract_doc_from_file(self, file_path):
        """ -> string """

        file_contents = open(file_path).read()
        lexer = Lexer(file_contents, None)
        tokens = lexer.tokenize()
        parser = Parser(lexer.tokenize())

        index = 0
        result = ""

        for node in parser.parse():
            if isinstance(node, CommentNode):
                result = tokens[index+1].contents.strip()
                break

            index += 1

        return result


    def parse_doc(self, doc):
        """
        :doc: string
        parses the given string and returns a dict with all documentation tags
        -> dict
        """

        lines = doc.split("\n")
        current_tag = None
        ret = {}

        for line in lines:
            line = line.strip()

            if line.startswith('@doc'):
                continue

            if not line:
                continue

            if line and line[0] == '@':
                words = line.split(' ')
                current_tag = words[0][1:]
                ret[current_tag] = " ".join(words[1:]).strip()

            elif current_tag in ret:
                if ret[current_tag] != '':
                    ret[current_tag] += "\n"

                ret[current_tag] += "%s" % line

        return ret


# -*- coding: utf-8 -*-
 
from django.shortcuts import render
from django.core.cache import cache
from styleguide.utils import (Styleguide, STYLEGUIDE_DIR_NAME, STYLEGUIDE_DEBUG,
                              STYLEGUIDE_CACHE_NAME)


def index(request, module_name=None, component_name=None):
    styleguide = None
    if not STYLEGUIDE_DEBUG:
        styleguide = cache.get(STYLEGUIDE_CACHE_NAME)

    if styleguide is None:
        styleguide = Styleguide()
        cache.set(STYLEGUIDE_CACHE_NAME, styleguide, None)

    if module_name is not None:
        styleguide.set_current_module(module_name)

    context = { 'styleguide': styleguide }
    index_path = "%s/index.html" % STYLEGUIDE_DIR_NAME
    return render(request, index_path, context)



# -*- coding: utf-8 -*-
from distutils.core import setup
from setuptools import find_packages

setup(
    name='django-styleguide',
    version='0.1.13',
    author=u'André Farzat',
    author_email='andrefarzat@gmail.com',
    packages=find_packages(),
    url='http://pypi.python.org/pypi/django-styleguide/',
    license='LICENSE',
    description='Styleguide for django projects',
    long_description=open('README.md').read(),
    install_requires=["Django >= 1.4",],
    include_package_data=True,
    zip_safe=False,
)



# -*- coding: utf-8 -*-

import unittest
import os

from django.core.urlresolvers import reverse

from .utils import StyleguideLoader, Styleguide, STYLEGUIDE_DIR_NAME


CURRENT_PATH = os.path.dirname(__file__)
PROJECT_ROOT = os.path.join(CURRENT_PATH, '..')
MOCK_PROJECT_PATH = os.path.join(PROJECT_ROOT, 'styleguide_mock', 'templates')
STYLEGUIDE_URL = reverse("styleguide.index")

DOC_STRING = """@doc

@name layout area
@description Nothing more than an area


""".strip()

DOC_STRING_2 = """@doc

@name something different
@oneline does this work ?
@twolines if I put this in
twolines, would this work ?
@threelines
  Now I am getting too deep ?
  I am going to write three lines !
  OMG! I did wrote three lines !!!

""".strip()


class StyleguideLoaderTest(unittest.TestCase):

    def setUp(self):
        self.loader = StyleguideLoader()


    def test_format_name(self):

        # [ (given_file_name, expected_result), ... ]
        file_name_list_to_be_tested = [
            ("01-name.html", "name"),
            ("0-name.html", "0-name"),
            ("0x-name.html", "0x-name"),
            ("89-name.html", "name"),
            ("name.nothing.html", "name.nothing"),
            ("nothing.todo.no-where-to-go", "nothing.todo.no-where-to-go"),

            # replacing underscore to whitespace
            ("name_separated", "name separated"),
            ("1_2_3_4_5_6", "1 2 3 4 5 6"),
            ('____A', "    A"),
        ]


        for file_name, expected_result in file_name_list_to_be_tested:
            formatted_file_name = self.loader._format_file_name(file_name)
            self.assertEqual(formatted_file_name, expected_result)


    def test__get_components_from_folder(self):
        expected_result = [
            { 'id': u'footer', 'file_name': u'footer.html', 'name': u'footer', 'template': u'styleguide/layout/footer.html', 'doc': {}, 'link': STYLEGUIDE_URL+'layout#footer' },
            { 'id': u'header', 'file_name': u'header.html', 'name': u'header', 'template': u'styleguide/layout/header.html', 'doc': {}, 'link': STYLEGUIDE_URL+'layout#header' }
        ]

        path_to_test = os.path.join(MOCK_PROJECT_PATH, STYLEGUIDE_DIR_NAME)
        result = self.loader._get_components_from_folder(path_to_test, 'layout')

        self.assertEqual(result, expected_result)


    def test_extract_doc_from_file(self):
        expected_result = DOC_STRING

        path_to_file = os.path.join(MOCK_PROJECT_PATH, STYLEGUIDE_DIR_NAME, 'components', '02-area.html')
        result = self.loader.extract_doc_from_file(path_to_file)

        self.assertEqual(result, expected_result)


    def test_parse_doc(self):
        expected_result = {
            'name': 'layout area',
            'description': 'Nothing more than an area'
        }

        result = self.loader.parse_doc(DOC_STRING)
        self.assertEqual(result, expected_result)

        expected_result = {
            "name": "something different",
            "oneline": "does this work ?",
            "twolines": "if I put this in\ntwolines, would this work ?",
            "threelines": "Now I am getting too deep ?\nI am going to write three lines !\nOMG! I did wrote three lines !!!"
        }

        result = self.loader.parse_doc(DOC_STRING_2)
        self.assertEqual(result, expected_result)


class StyleguideTest(unittest.TestCase):

    def setUp(self):
        self.styleguide = Styleguide()


    def test_modules(self):
        expected_result = ['components', 'layout']
        result = [m.name for m in self.styleguide.modules]
        self.assertEqual(result, expected_result)


    def test_components(self):
        expected_result = ['bar', 'layout area', 'footer', 'header']
        result = [c.name for c in self.styleguide.components]
        self.assertEqual(result, expected_result)


    def test_modules_components(self):
        expected_result = ['bar', 'layout area']
        result = [c.name for c in self.styleguide.modules[0].components]
        self.assertEqual(result, expected_result)

        expected_result = ['footer', 'header']
        result = [c.name for c in self.styleguide.modules[1].components]
        self.assertEqual(result, expected_result)


    def test_set_current_module(self):
        # Begins None
        self.assertEqual(self.styleguide.current_module, None)
        # Begins an empty list
        self.assertEqual(self.styleguide.current_components, [])
        # And we are in the index page
        self.assertEqual(self.styleguide.is_index(), True)

        expected_result = self.styleguide.modules[0]
        self.styleguide.set_current_module('layout')
        self.assertEqual(self.styleguide.current_module, expected_result)

        # current_components
        expected_result = self.styleguide.modules[0].components
        self.assertEqual(self.styleguide.current_components, expected_result)

        # Not index page anymore
        self.assertEqual(self.styleguide.is_index(), False)




